unit RIGON_SHExt;

interface

uses
  Winapi.ActiveX, System.Win.ComObj, System.Win.Comserv, Winapi.Windows, System.SysUtils,
  System.Classes, Winapi.ShlObj, Vcl.DdeMan;

type
  TShellExtension = class(TComObject, IShellExtInit, IContextMenu)
  private
    FFiles: string;
    procedure ExecuteServer(const Service, Topic: string);
  public
    //IShellExtInit
    function Initialize(pidlFolder: PItemIDList; lpdObj: IDataObject;
      hKeyProgID: HKEY): HResult; reintroduce; stdcall;
    //IContextMenu
    function QueryContextMenu(Menu: HMENU;
      indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult; stdcall;
    function InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult; stdcall;
    function GetCommandString(idCmd: UINT_PTR; uFlags: UINT; pwReserved: PUINT;
      pszName: LPSTR; cchMax: UINT): HResult; stdcall;
  end;

  TShellExtensionFactory = class (TComObjectFactory)
  public
    procedure UpdateRegistry(Register: Boolean); override;
  end;

implementation

uses
  RIGON_Consts,
  Winapi.ShellAPI, System.Win.Registry, System.AnsiStrings, AppData, RegData, SysInfo;

{ TShellExtension }

function TShellExtension.Initialize(pidlFolder: PItemIDList; lpdObj: IDataObject;
  hKeyProgID: HKEY): HResult;
var
  FormatEtc: TFormatEtc;
  StgMedium: TStgMedium;
  Buffer: TStrings;
  Count, i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  Result := E_FAIL;
  if lpdObj = nil then Exit;

  with FormatEtc do
  begin
    cfFormat := CF_HDROP;
    ptd := nil;
    dwAspect := DVASPECT_CONTENT;
    lindex := -1;
    tymed := TYMED_HGLOBAL;
  end;
  Result := lpdObj.GetData(FormatEtc, StgMedium);
  if Succeeded(Result) then
  begin
    Buffer := TStringList.Create;
    try
      Count := DragQueryFile(StgMedium.hGlobal, $FFFFFFFF, nil, 0);
      for i := 0 to Count - 1 do
      begin
        DragQueryFile(StgMedium.hGlobal, i, FileName, SizeOf(FileName));
        Buffer.Add(FileName);
      end;
      FFiles := Buffer.Text;
    finally
      Buffer.Free;
      ReleaseStgMedium(StgMedium);
    end;
  end;
end;

function TShellExtension.QueryContextMenu(Menu: HMENU;
  indexMenu, idCmdFirst, idCmdLast, uFlags: UINT): HResult;
const
  MF_STRBYPOS = MF_STRING or MF_BYPOSITION;
var
  Reg: TRegistry;
begin
  Result := 0;
  if (uFlags and CMF_NORMAL) <> CMF_NORMAL then Exit;

  Reg := TRegistry.Create;
  try
    if Reg.OpenKeyReadOnly('Software\Classes\CLSID\' + STR_ShellExtension + '\InprocServer32') then
    begin
      //! InsertMenu(Menu, idCmdFirst, MF_SEPARATOR or MF_BYPOSITION, 0, nil);
      if Reg.ValueExists('SendToFolder') then
      begin
        if InsertMenu(Menu, indexMenu +1, MF_STRBYPOS, idCmdFirst, PChar(SSTFCaption)) then
          Inc(Result);
      end;
      if Reg.ValueExists('SendToClipboard') then
      begin
        if InsertMenu(Menu, indexMenu + 2, MF_STRBYPOS, idCmdFirst + 1, PChar(SSTCCaption)) then
          Inc(Result);
      end;
      if Reg.ValueExists('DatetimeAndAttribute') then
      begin
        if InsertMenu(Menu, indexMenu + 3, MF_STRBYPOS, idCmdFirst + 2, PChar(SDAACaption)) then
          Inc(Result);
      end;
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TShellExtension.ExecuteServer(const Service, Topic: string);
const
  SEE_MASK_SYNCIDLE = SEE_MASK_NOASYNC or SEE_MASK_WAITFORINPUTIDLE;
var
  Reg: TRegistry;
  Path, Err: string;
  ExecuteInfo: TShellExecuteInfo;
  DdeClient: TDdeClientConv;
  Command: TStrings;
begin
  Reg := TRegistry.Create;
  try
    if Reg.OpenKeyReadOnly('Software\Classes\CLSID\' + STR_ShellExtension + '\InprocServer32') then
    begin
      Path := Reg.ReadString(Service);
      Reg.CloseKey;
    end
    else
      Path := '';
  finally
    Reg.Free;
  end;
  Err := '';

  FillChar(ExecuteInfo, SizeOf(TShellExecuteInfo), 0);
  with ExecuteInfo do begin
    cbSize := SizeOf(ExecuteInfo);
    fMask := SEE_MASK_FLAG_NO_UI or SEE_MASK_SYNCIDLE {or SEE_MASK_NOCLOSEPROCESS};
    lpVerb := 'open';
    lpFile := PChar(Format('%s\%s.exe', [Path, Service]));
    nShow := SW_SHOWNORMAL;
  end;
  if ShellExecuteEx(@ExecuteInfo) then
  begin
    DdeClient := TDdeClientConv.Create(nil);
    try
      if DdeClient.SetLink(Service, Topic) then
      begin
        Command := TStringList.Create;
        try
          Command.Text := FFiles;
          if not DdeClient.ExecuteMacroLines(Command, False) then
            Err := 'ExecuteMacroLines';
        finally
          Command.Free;
        end;
      end
      else
        Err := 'SetLink';
    finally
      DdeClient.Free;
    end;
  end
  else
    Err := 'ShellExecuteEx';

  if Err <> '' then
    MessageBox(0, PChar('FAIL: ExecuteServer.' + Err), 'Right-on', MB_ICONERROR + MB_OK);
end;

function TShellExtension.InvokeCommand(var lpici: TCMInvokeCommandInfo): HResult;
begin
  if HiWord(Integer(lpici.lpVerb)) <> 0 then
  begin
    Result := E_FAIL;
    Exit;
  end
  else
    Result := NOERROR;

  case LoWord(Integer(lpici.lpVerb)) of
    0: ExecuteServer('SendToFolder', 'DdeServerConv');
    1: ExecuteServer('SendToClipboard', 'DdeServerConv');
    2: ExecuteServer('DatetimeAndAttribute', 'DdeServerConv');
  else
    Result := E_INVALIDARG;
    MessageBox(0, 'FAIL: IContextMenu.InvokeCommand', 'Right-on', MB_ICONERROR + MB_OK);
  end;
end;

function TShellExtension.GetCommandString(idCmd: UINT_PTR; uFlags: UINT; pwReserved: PUINT;
  pszName: LPSTR; cchMax: UINT): HResult;
begin
  Result := NOERROR;
  case idCmd of
    0:  //STF
    begin
      case uFlags of
        GCS_VERBA: System.AnsiStrings.StrLCopy(pszName, PAnsiChar(AnsiString(SSTFHint)), cchMax - 1);
        GCS_VERBW: System.SysUtils.StrLCopy(PWideChar(pszName), PWideChar(SSTFHint), cchMax - 1);
      end;
    end;
    1:  //STC
    begin
      case uFlags of
        GCS_VERBA: System.AnsiStrings.StrLCopy(pszName, PAnsiChar(AnsiString(SSTCHint)), cchMax - 1);
        GCS_VERBW: System.SysUtils.StrLCopy(PWideChar(pszName), PWideChar(SSTCHint), cchMax - 1);
      end;
    end;
    2:  //DAA
    begin
      case uFlags of
        GCS_VERBA: System.AnsiStrings.StrLCopy(pszName, PAnsiChar(AnsiString(SDAAHint)), cchMax - 1);
        GCS_VERBW: System.SysUtils.StrLCopy(PWideChar(pszName), PWideChar(SDAAHint), cchMax - 1);
      end;
    end
  else
    Result := E_INVALIDARG;
  end;
end;

{ TShellExtensionFactory }

procedure TShellExtensionFactory.UpdateRegistry(Register: Boolean);
var
  Reg: TRegData;
  Ini: TAppIniFile;
  Path: string;
begin
  Reg := TRegData.Create;
  try
    if Register then
    begin
      if Reg.OpenKey('Software\Classes\*\shellex\ContextMenuHandlers\' + KeyName, True) then
      begin
        Reg.WriteString('', STR_ShellExtension);
        Reg.CloseKey;
      end;

      //'lnkfile' `ȂƃR}hȂB'*' Ɋ܂܂ȂH
      if Reg.OpenKey('Software\Classes\lnkfile\shellex\ContextMenuHandlers\' + KeyName, True) then
      begin
        Reg.WriteString('', STR_ShellExtension);
        Reg.CloseKey;
      end;

      if Reg.OpenKey('Software\Classes\Folder\shellex\ContextMenuHandlers\' + KeyName, True) then
      begin
        Reg.WriteString('', STR_ShellExtension);
        Reg.CloseKey;
      end;

      if Reg.OpenKey('Software\Classes\CLSID\' + STR_ShellExtension, True) then
      begin
        Reg.WriteString('', 'RightOn');
        if Reg.OpenKey('InprocServer32', True) then
        begin
          Reg.ClearValues;
          Reg.WriteString('', ModuleFileName);
          Reg.WriteString('ThreadingModel', 'Apartment');

          Ini := TAppIniFile.Create(AppDataName('ShaggyDog\RightOn\RightOn.ini'));
          try
            if Ini.ReadBool('SendToFolder', 'Enabled', False) then
            begin
              Path := Ini.ReadString('SendToFolder', 'Path', '');
              if Path = '' then
                Path := ExtractFilePath(ModuleFileName);
              Reg.WriteString('SendToFolder', Path);
            end;
            if Ini.ReadBool('SendToClipboard', 'Enabled', False) then
            begin
              Path := Ini.ReadString('SendToClipboard', 'Path', '');
              if Path = '' then
                Path := ExtractFilePath(ModuleFileName);
              Reg.WriteString('SendToClipboard', Path);
            end;
            if Ini.ReadBool('DatetimeAndAttribute', 'Enabled', False) then
            begin
              Path := Ini.ReadString('DatetimeAndAttribute', 'Path', '');
              if Path = '' then
                Path := ExtractFilePath(ModuleFileName);
              Reg.WriteString('DatetimeAndAttribute', Path);
            end;
          finally
            Ini.Free;
          end;
        end;
        Reg.CloseKey;
      end;
    end
    else
    begin
      if Reg.OpenKey('Software\Classes', True) then
      begin
        Reg.DeletePath('*\shellex\ContextMenuHandlers\RightOn');
        Reg.DeletePath('lnkfile\shellex\ContextMenuHandlers\RightOn');
        Reg.DeletePath('Folder\shellex\ContextMenuHandlers\RightOn');
        Reg.CloseKey;
      end;

      Reg.DeleteKey('Software\Classes\CLSID\' + STR_ShellExtension);
    end;
  finally
    Reg.Free;
  end;
end;

initialization
  TShellExtensionFactory.Create(ComServer, TShellExtension, GUID_ShellExtension,
    'RightOn', 'RIGON_SHExt', ciMultiInstance, tmApartment);

end.
