unit RIGON_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ExtCtrls, RefCtrls;

type
  TMainForm = class(TForm)
    OKButton: TButton;
    CancelButton: TButton;
    SendToFolderPanel: TPanel;
    SendToFolderChk: TCheckBox;
    SendToFolderLbl: TLabel;
    SendToFolderEdt: TRefDirectoryEdit;
    SendToClipboardPanel: TPanel;
    SendToClipboardChk: TCheckBox;
    SendToClipboardLbl: TLabel;
    SendToClipboardEdt: TRefDirectoryEdit;
    RunWithParameterPanel: TPanel;
    RunWithParameterChk: TCheckBox;
    RunWithParameterLbl: TLabel;
    RunWithParameterEdt: TRefDirectoryEdit;
    PropertyOfLinkPanel: TPanel;
    PropertyOfLinkChk: TCheckBox;
    PropertyOfLinkLbl: TLabel;
    PropertyOfLinkEdt: TRefDirectoryEdit;
    DatetimeAndAttributePanel: TPanel;
    DatetimeAndAttributeChk: TCheckBox;
    DatetimeAndAttributeLbl: TLabel;
    DatetimeAndAttributeEdt: TRefDirectoryEdit;
    procedure FormCreate(Sender: TObject);
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
  private
    { Private 錾 }
    procedure ReadOption;
    procedure WriteOption;
    procedure UpdateOption;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  RIGON_Consts,
  RegData, AppData, Winapi.ShellAPI, System.UITypes, SysInfo, System.IOUtils;

procedure TMainForm.ReadOption;
var
  ExePath: string;
  Reg: TRegData;

  function ExtractCommandPath(Command: string): string;
  var
    p: PChar;
  begin
    Result := '';
    p := PChar(Command);
    Result := ExtractFileDir(AnsiExtractQuotedStr(p, '"'));
  end;

begin
  ExePath := ExtractFileDir(ParamStr(0));
  SendToFolderEdt.Dialog.Directory := ExePath;
  SendToClipboardEdt.Dialog.Directory := ExePath;
  RunWithParameterEdt.Dialog.Directory := ExePath;
  PropertyOfLinkEdt.Dialog.Directory := ExePath;
  DatetimeAndAttributeEdt.Dialog.Directory := ExePath;

  Reg := TRegData.Create;
  try
    if Reg.OpenKeyReadOnly('Software\Classes\CLSID\' + STR_ShellExtension + '\InprocServer32') then
    begin
      if Reg.ValueExists('SendToFolder') then
      begin
        SendToFolderChk.Checked := True;
        SendToFolderEdt.Text := Reg.ReadString('SendToFolder');
      end;
      if Reg.ValueExists('SendToClipboard') then
      begin
        SendToClipboardChk.Checked := True;
        SendToClipboardEdt.Text := Reg.ReadString('SendToClipboard');
      end;
      if Reg.ValueExists('DatetimeAndAttribute') then
      begin
        DatetimeAndAttributeChk.Checked := True;
        DatetimeAndAttributeEdt.Text := Reg.ReadString('DatetimeAndAttribute');
      end;
      Reg.CloseKey;
    end;

    if Reg.OpenKeyReadOnly('Software\Classes\*\shell\RunWithParameter\command') then
    begin
      RunWithParameterChk.Checked := True;
      RunWithParameterEdt.Text := ExtractCommandPath(Reg.ReadString(''));
      Reg.CloseKey;
    end;

    if Reg.OpenKeyReadOnly('Software\Classes\lnkfile\shell\PropertyOfLink\command') then
    begin
      PropertyOfLinkChk.Checked := True;
      PropertyOfLinkEdt.Text := ExtractCommandPath(Reg.ReadString(''));
      Reg.CloseKey;
    end;
  finally
    Reg.Free;
  end;
end;

procedure TMainForm.WriteOption;
var
  Ini: TAppIniFile;
begin
  Ini := TAppIniFile.Create(AppDataName('ShaggyDog\RightOn\RightOn.ini'));
  try
    Ini.WriteBool('SendToFolder', 'Enabled', SendToFolderChk.Checked);
    Ini.WriteString('SendToFolder', 'Path', SendToFolderEdt.Text);

    Ini.WriteBool('SendToClipboard', 'Enabled', SendToClipboardChk.Checked);
    Ini.WriteString('SendToClipboard', 'Path', SendToClipboardEdt.Text);

    Ini.WriteBool('RunWithParameter', 'Enabled', RunWithParameterChk.Checked);
    Ini.WriteString('RunWithParameter', 'Path', RunWithParameterEdt.Text);

    Ini.WriteBool('PropertyOfLink', 'Enabled', PropertyOfLinkChk.Checked);
    Ini.WriteString('PropertyOfLink', 'Path', PropertyOfLinkEdt.Text);

    Ini.WriteBool('DatetimeAndAttribute', 'Enabled', DatetimeAndAttributeChk.Checked);
    Ini.WriteString('DatetimeAndAttribute', 'Path', DatetimeAndAttributeEdt.Text);
  finally
    Ini.Free;
  end;
end;

procedure TMainForm.UpdateOption;

  procedure RegistShellex(FileName: string; Entry: Boolean);
  var
    Parameters: string;
  begin
    if FileExists(FileName) then
    begin
      if Entry then
        Parameters := Format('/s %s', [AnsiQuotedStr(FileName, '"')])
      else
        Parameters := Format('/s /u %s', [AnsiQuotedStr(FileName, '"')]);
      ShellExecute(Handle, 'open', 'regsvr32', PChar(Parameters), '', SW_SHOWNORMAL);
    end
    else
      MessageDlg(Format(SToolNotFound, [AnsiQuotedStr(FileName, '"')]), mtError, [mbOk], 0);
  end;

  function BuildCommandStr(Path, FileName: string): string;
  begin
    if Path = '' then
      Path := ExtractFileDir(ParamStr(0));

    Result := Format('%s %s', [AnsiQuotedStr(TPath.Combine(Path, FileName), '"'), '"%1"']);
  end;

var
  Entrance: Boolean;
  Reg: TRegData;
begin
  Entrance := SendToFolderChk.Checked or SendToClipboardChk.Checked or DatetimeAndAttributeChk.Checked;
  RegistShellex(ChangeFileExt(ParamStr(0), '.dll'), Entrance);
  if IsWin64 then
    RegistShellex(ChangeFileExt(ParamStr(0), '.x64.dll'), Entrance);

  Reg := TRegData.Create;
  try
    if RunWithParameterChk.Checked then
    begin
      if Reg.OpenKey('Software\Classes\*\shell\RunWithParameter', True) then
      begin
        Reg.WriteString('', SRWPCaption);
        if Reg.OpenKey('command', True) then
        begin
          Reg.WriteString('', BuildCommandStr(RunWithParameterEdt.Text, 'RunWithParameter.exe'));
          Reg.CloseKey;
        end;
        Reg.CloseKey;
      end;
      if Reg.OpenKey('Software\Classes\Folder\shell\RunWithParameter', True) then
      begin
        Reg.WriteString('', SRWPCaption);
        if Reg.OpenKey('command', True) then
        begin
          Reg.WriteString('', BuildCommandStr(RunWithParameterEdt.Text, 'RunWithParameter.exe'));
          Reg.CloseKey;
        end;
        Reg.CloseKey;
      end;
    end
    else
    begin
      if Reg.OpenKey('Software\Classes', False) then
      begin
        Reg.DeletePath('*\shell\RunWithParameter');
        Reg.DeletePath('Folder\shell\RunWithParameter');
        Reg.CloseKey;
      end;
    end;

    if PropertyOfLinkChk.Checked then
    begin
      if Reg.OpenKey('Software\Classes\.lnk', True) then
      begin
        Reg.WriteString('', 'lnkfile');
        Reg.CloseKey;
      end;

      if Reg.OpenKey('Software\Classes\lnkfile\shell\PropertyOfLink', True) then
      begin
        Reg.WriteString('', SPOLCaption);
        if Reg.OpenKey('command', True) then
        begin
          Reg.WriteString('', BuildCommandStr(PropertyOfLinkEdt.Text, 'PropertyOfLink.exe'));
          Reg.CloseKey;
        end;
      end;
    end
    else
    begin
      if Reg.OpenKey('Software\Classes', False) then
      begin
        Reg.DeletePath('lnkfile\shell\PropertyOfLink');
        Reg.CloseKey;
      end;

      if not Reg.KeyExists('Software\Classes\lnkfile') then
        Reg.DeleteKey('Software\Classes\.lnk');
    end;
  finally
    Reg.Free;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ReadOption;
end;

procedure TMainForm.OKButtonClick(Sender: TObject);
begin
  WriteOption;
  UpdateOption;

  Close;
end;

procedure TMainForm.CancelButtonClick(Sender: TObject);
begin
  Close;
end;

end.
