(* FileUtils

Ver, 1.00 2013/04/24 ꉞB
Ver, 1.01 2013/08/28 HasSymLink ǉBFileTimeToDateTime, StrToDateTime  FileTimeToLocalDateTime, StrToLocalDateTime ɉB
Ver, 1.02 2016/10/05 HasDirectory, HasSymLink gB
Ver, 1.03 2021/04/15 ZeroFileTime, IsValidFileTime, LocalDateTimeToFileTime, SetTimeStamp ǉB

I/O ֘A[eBWB
FileProc p~ IOUtils gn߂񂾂ǁAꂾł͎܂ȂȂČǕB

  IOUtils+ *)

unit FileUtils;

interface

uses
  Winapi.Windows, System.SysUtils, System.StrUtils, System.Masks;

function MatchesMasks(const FileName, Masks: string): Boolean;
function MatchesCompoundMasks(const FileName, Includes, Excludes: string): Boolean;
function HasDirectory(Rec: TSearchRec): Boolean; overload;
function HasDirectory(const FileName: string): Boolean; overload;
function HasSymLink(Rec: TSearchRec): Boolean; overload;
function HasSymLink(const FileName: string): Boolean; overload;
function FileTimeToLocalDateTime(UTCFileTime: TFileTime): TDateTime;
function StrToLocalDateTime(S: string): TDateTime;
procedure ZeroFileTime(var AFileTime: TFileTime);
function IsValidFileTime(AFileTime: TFileTime): Boolean;
function LocalDateTimeToFileTime(LocalDateTime: TDateTime): TFileTime;
function SetTimeStamp(FileName: string;
  CreationTime, LastAccessTime, LastWriteTime: PFileTime): Boolean;

implementation

//FileName  Masks ɓK邩`FbNB
//Masks ̓p^[ ';' ŘAB
function MatchesMasks(const FileName, Masks: string): Boolean;
var
  Mask: string;
begin
  Result := False;
  for Mask in SplitString(Masks, ';') do
  begin
    Result := MatchesMask(FileName, Mask);
    if Result then Break;
  end;
end;

//FileName  Includes ɓKAExcludes ɓKȂ`FbNB
function MatchesCompoundMasks(const FileName, Includes, Excludes: string): Boolean;
begin
  Result := MatchesMasks(FileName, Includes) and not MatchesMasks(FileName, Excludes);
end;

//w背R[hfBNg`FbNB
function HasDirectory(Rec: TSearchRec): Boolean;
begin
  Result := (Rec.Attr and faDirectory) <> 0;
end;

//wt@CfBNg`FbNB
function HasDirectory(const FileName: string): Boolean;
var
  Rec: TSearchRec;
begin
  Result := FindFirst(FileName, faAnyFile, Rec) = 0;
  try
    if Result then
      Result := HasDirectory(Rec);
  finally
    FindClose(Rec);
  end;
end;

//w背R[hV{bNN`FbNB
function HasSymLink(Rec: TSearchRec): Boolean;
begin
  Result := (Rec.Attr and faSymLink) <> 0;
end;

//wt@CV{bNN`FbNB
function HasSymLink(const FileName: string): Boolean;
var
  Rec: TSearchRec;
begin
  Result := FindFirst(FileName, faAnyFile, Rec) = 0;
  try
    if Result then
      Result := HasSymLink(Rec);
  finally
    FindClose(Rec);
  end;
end;

//TFileTime ^̒l TDateTime ^ɕϊB
//UTC Ԃ烍[JԂւ̕ϊɍsB
function FileTimeToLocalDateTime(UTCFileTime: TFileTime): TDateTime;
var
  LocalFileTime: TFileTime;
  LocalSystemTime: TSystemTime;
begin
  FileTimeToLocalFileTime(UTCFileTime, LocalFileTime);
  FileTimeToSystemTime(LocalFileTime, LocalSystemTime);
  Result := SystemTimeToDateTime(LocalSystemTime);
end;

// TDateTime ^ɕϊB
//UTC Ԃ烍[JԂւ̕ϊɍsB
function StrToLocalDateTime(S: string): TDateTime;
var
  UTCDateTime: TDateTime;
  UTCSystemTime: TSystemTime;
  UTCFileTime: TFileTime;
begin
  UTCDateTime := StrToDateTime(S);
  DateTimeToSystemTime(UTCDateTime, UTCSystemTime);
  SystemTimeToFileTime(UTCSystemTime, UTCFileTime);
  Result := FileTimeToLocalDateTime(UTCFileTime);
end;

//wt@C^C[NAB
procedure ZeroFileTime(var AFileTime: TFileTime);
begin
  AFileTime.dwLowDateTime := 0;
  AFileTime.dwHighDateTime := 0;
end;

//wt@C^CL`FbNB
//l[ȂׂĂ邾Ȃ̂ŁAԂ̓e͕sB
function IsValidFileTime(AFileTime: TFileTime): Boolean;
begin
  Result := (AFileTime.dwLowDateTime <> 0) and (AFileTime.dwHighDateTime <> 0);
end;

//TDateTime ^̒l TFileTime ^ɕϊB
//[JԂ UTC Ԃւ̕ϊɍsB
function LocalDateTimeToFileTime(LocalDateTime: TDateTime): TFileTime;
var
  LocalFileTime: TFileTime;
  LocalSystemTime: TSystemTime;
begin
  ZeroFileTime(Result);

  DateTimeToSystemTime(LocalDateTime, LocalSystemTime);
  SystemTimeToFileTime(LocalSystemTime, LocalFileTime);
  LocalFileTimeToFileTime(LocalFileTime, Result);
end;

//wt@C̃^CX^v܂Ƃ߂ĕύXB
//ύXȂ nil [l̎ԂnB
function SetTimeStamp(FileName: string;
  CreationTime, LastAccessTime, LastWriteTime: PFileTime): Boolean;
var
  Handle: THandle;
begin
  Handle := CreateFile(PChar(FileName), GENERIC_WRITE, FILE_SHARE_READ, nil,
    OPEN_EXISTING, FILE_FLAG_BACKUP_SEMANTICS, 0);
  try
    Result := Handle <> INVALID_HANDLE_VALUE;
    if Result then
      Result := SetFileTime(Handle, CreationTime, LastAccessTime, LastWriteTime);
  finally
    CloseHandle(Handle);
  end;
end;

end.
