{*********************************************************}
{                                                         }
{       Delphi Visual Component Library           }
{                                                         }
// Copyright (c) 1995-2010 Embarcadero Technologies, Inc.
{                                                         }
{*********************************************************}
(* Tvt̂̂CEg

2014/02/05 {ꉻB
2020/10/02 R|[lgǉBقɂ悤ȁH

*)

unit ShellCtrlsProp;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Controls,
  Vcl.Forms, Vcl.Dialogs, DesignIntf, DesignEditors, FiltEdit, Vcl.StdCtrls, ShellConsts, ShellTypes,
  ShellCtrls, VistaShellCtrls;

type
  TRootPathEditDlg = class(TForm)
    btnOk: TButton;
    btnCancel: TButton;
    rbUseFolder: TRadioButton;
    gbFolder: TGroupBox;
    cbFolderType: TComboBox;
    gbPath: TGroupBox;
    ePath: TEdit;
    rbUsePath: TRadioButton;
    btnBrowse: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnCancelClick(Sender: TObject);
    procedure rbUseFolderClick(Sender: TObject);
    procedure rbUsePathClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
  private
    procedure UpdateState;
  public
  end;

  TRootProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TRootEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure Register;

implementation

{$R *.dfm}

uses
  System.TypInfo, Vcl.FileCtrl;

resourcestring
  SPickRootPath = '[gpXIĂ';
  SEditRoot = '[g̕ҏW(&D)...';

const
  NTFolders = [rfCommonDesktopDirectory, rfCommonPrograms, rfCommonStartMenu, rfCommonStartup];

function PathIsKFID(Value: string): Boolean;
begin
  Result := GetEnumValue(TypeInfo(TRootFolder), Value) >= 0;
end;

function RootPathEditor(Value : string): string;
begin
  Result := Value;
  with TRootPathEditDlg.Create(Application) do
    try
      rbUseFolder.Checked := PathIsKFID(Result);
      rbUsePath.Checked := not rbUseFolder.Checked;
      if not PathIsKFID(Result) then
      begin
        cbFolderType.ItemIndex := 0;
        ePath.Text := Result;
      end
      else
      begin
        cbFolderType.ItemIndex := cbFolderType.Items.IndexOf(Result);
      end;

      UpdateState;
      ShowModal;
      if ModalResult = mrOK then
      begin
        if rbUsePath.Checked then
          Result := ePath.Text
        else
          Result := cbFolderType.Items[cbFolderType.ItemIndex];
      end;
    finally
      Free;
    end;
end;

procedure TRootProperty.Edit;
begin
  SetStrValue(RootPathEditor(GetStrValue));
end;

function TRootProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

procedure TRootPathEditDlg.FormCreate(Sender: TObject);
var
  FT: TRootFolder;
begin
  for FT := Low(TRootFolder) to High(TRootFolder) do
    if not ((Win32Platform = VER_PLATFORM_WIN32_WINDOWS) and (FT in NTFolders)) then
      cbFolderType.Items.Add(GetEnumName(TypeInfo(TRootFolder), Ord(FT)));
  cbFolderType.ItemIndex := 0;
end;

procedure TRootPathEditDlg.UpdateState;
begin
  cbFolderType.Enabled := rbUseFolder.Checked;
  ePath.Enabled := not rbUseFolder.Checked;
  btnBrowse.Enabled := ePath.Enabled;
end;

procedure TRootPathEditDlg.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TRootPathEditDlg.btnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel;
  Close;
end;

procedure TRootPathEditDlg.rbUseFolderClick(Sender: TObject);
begin
  rbUsePath.Checked := not rbUseFolder.Checked;
  UpdateState;
end;

procedure TRootPathEditDlg.rbUsePathClick(Sender: TObject);
begin
  rbUseFolder.Checked := not rbUsePath.Checked;
  UpdateState;
end;

procedure TRootPathEditDlg.btnOkClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TRootPathEditDlg.btnBrowseClick(Sender: TObject);
var
  Path : WideString;
  Dir : string;
begin
  Path := ePath.Text;
  Dir := ePath.Text;
  if SelectDirectory(SPickRootPath, '', Dir) then
    ePath.Text := Dir;
end;

{ TRootEditor }

procedure TRootEditor.ExecuteVerb(Index: Integer);

  procedure EditRoot;
  const
    SRoot = 'root';
  var
    Path : string;
  begin
    Path := RootPathEditor(GetPropValue(Component, SRoot, True));
    SetPropValue(Component, SRoot, Path);
    Designer.Modified;
  end;

begin
  case Index of
    0: EditRoot;
  end;
end;

function TRootEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := SEditRoot;
  end;
end;

function TRootEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TRoot), TShellTreeView, SRootPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TShellComboBox, SRootPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TShellListView, SRootPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TShellChangeNotifier, SRootPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TExplorerBrowser, SPathPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TNamespaceTreeRoot, SPathPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TBreadcrumbComboItem, SPathPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(TRoot), TBreadcrumbComboBox, SPathPropertyName, TRootProperty);
  RegisterPropertyEditor(TypeInfo(string), TFilterComboBoxEx, 'Filter', TFilterProperty);
  RegisterComponentEditor(TShellTreeView, TRootEditor);
  RegisterComponentEditor(TShellListView, TRootEditor);
  RegisterComponentEditor(TShellComboBox, TRootEditor);
  RegisterComponentEditor(TShellChangeNotifier, TRootEditor);
  RegisterComponentEditor(TExplorerBrowser, TRootEditor);
  RegisterComponentEditor(TNameSpaceTreeControl, TRootEditor);
  RegisterComponentEditor(TBreadcrumbComboBox, TRootEditor);
end;

end.
