(* ShellLink

Ver, 1.00 2012/02/12 TShellLink, GetLinkTarget B
Ver, 1.01 2012/02/22 string ^̕Ԓl̒B
Ver, 1.02 2018/11/22 TShellLink Đ݌vBMakeLink, MakeSymlink ǉB

   VFN[`W *)

unit ShellLink;

interface

uses
  Winapi.Windows, System.SysUtils, Winapi.ShlObj, Winapi.ActiveX, System.Win.ComObj;

const
  { IPersistFile.Load dwMode }
  STGM_READ             = Winapi.ActiveX.STGM_READ;
  STGM_WRITE            = Winapi.ActiveX.STGM_WRITE;
  STGM_READWRITE        = Winapi.ActiveX.STGM_READWRITE;
  STGM_SHARE_DENY_NONE  = Winapi.ActiveX.STGM_SHARE_DENY_NONE;
  STGM_SHARE_DENY_READ  = Winapi.ActiveX.STGM_SHARE_DENY_READ;
  STGM_SHARE_DENY_WRITE = Winapi.ActiveX.STGM_SHARE_DENY_WRITE;
  STGM_SHARE_EXCLUSIVE  = Winapi.ActiveX.STGM_SHARE_EXCLUSIVE;
  STGM_PRIORITY         = Winapi.ActiveX.STGM_PRIORITY;
  STGM_CREATE           = Winapi.ActiveX.STGM_CREATE;
  STGM_CONVERT          = Winapi.ActiveX.STGM_CONVERT;
  STGM_FAILIFTHERE      = Winapi.ActiveX.STGM_FAILIFTHERE;
  STGM_DIRECT           = Winapi.ActiveX.STGM_DIRECT;
  STGM_TRANSACTED       = Winapi.ActiveX.STGM_TRANSACTED;
  STGM_NOSCRATCH        = Winapi.ActiveX.STGM_NOSCRATCH;
  STGM_NOSNAPSHOT       = $00200000;
  STGM_SIMPLE           = Winapi.ActiveX.STGM_SIMPLE;
  STGM_DIRECT_SWMR      = Winapi.ActiveX.STGM_DIRECT_SWMR;
  STGM_DELETEONRELEASE  = Winapi.ActiveX.STGM_DELETEONRELEASE;

  { IShellLink.Resolve fFlags }
  SLR_NO_UI                     = Winapi.ShlObj.SLR_NO_UI;
  SLR_ANY_MATCH                 = Winapi.ShlObj.SLR_ANY_MATCH;
  SLR_UPDATE                    = Winapi.ShlObj.SLR_UPDATE;
  SLR_NOUPDATE                  = Winapi.ShlObj.SLR_NOUPDATE;
  SLR_NOSEARCH                  = Winapi.ShlObj.SLR_NOSEARCH;
  SLR_NOTRACK                   = Winapi.ShlObj.SLR_NOTRACK;
  SLR_NOLINKINFO                = Winapi.ShlObj.SLR_NOLINKINFO;
  SLR_INVOKE_MSI                = Winapi.ShlObj.SLR_INVOKE_MSI;
  SLR_NO_UI_WITH_MSG_PUMP       = Winapi.ShlObj.SLR_NO_UI_WITH_MSG_PUMP;
  SLR_OFFER_DELETE_WITHOUT_FILE = Winapi.ShlObj.SLR_OFFER_DELETE_WITHOUT_FILE;
  SLR_KNOWNFOLDER               = Winapi.ShlObj.SLR_KNOWNFOLDER;
  SLR_MACHINE_IN_LOCAL_TARGET   = Winapi.ShlObj.SLR_MACHINE_IN_LOCAL_TARGET;
  SLR_UPDATE_MACHINE_AND_SID    = Winapi.ShlObj.SLR_UPDATE_MACHINE_AND_SID;


type
  { IShellLink bp[ }
  TShellLink = class
  private
    { Private 錾 }
    FParentWnd: HWND;
    FShellLink: IShellLink;
    FPersistFile: IPersistFile;
    function GetArguments: string;
    procedure SetArguments(Value: string);
    function GetDescription: string;
    procedure SetDescription(Value: string);
    function GetHotkey: Word;
    procedure SetHotkey(Value: Word);
    function GetIconPath: string;
    procedure SetIconPath(Value: string);
    function GetIconIndex: Integer;
    procedure SetIconIndex(Value: Integer);
    function GetIDList: PItemIDList;
    procedure SetIDList(Value: PItemIDList);
    function GetTargetPath: string;
    procedure SetTargetPath(Value: string);
    function GetShowCmd: Integer;
    procedure SetShowCmd(Value: Integer);
    function GetWorkingDirectory: string;
    procedure SetWorkingDirectory(Value: string);
  public
    { Public 錾 }
    constructor Create(Parent: HWND);
    destructor Destroy; override;
    function Resolve(Flags: Cardinal): Boolean;
    function Load(FileName: string; Mode: Cardinal): Boolean;
    function Save(FileName: string; Remember: Boolean): Boolean;
    function ResolvingLoad(FileName: string; Mode, Flags: Cardinal): Boolean;
    function ResolvingSave(FileName: string; Remember: Boolean; Flags: Cardinal): Boolean;
    property Arguments: string read GetArguments write SetArguments;
    property Description: string read GetDescription write SetDescription;
    property Hotkey: Word read GetHotkey write SetHotkey;
    property IconPath: string read GetIconPath write SetIconPath;
    property IconIndex: Integer read GetIconIndex write SetIconIndex;
    property IDList: PItemIDList read GetIDList write SetIDList;
    property TargetPath: string read GetTargetPath write SetTargetPath;
    property ShowCmd: Integer read GetShowCmd write SetShowCmd;
    property WorkingDirectory: string read GetWorkingDirectory write SetWorkingDirectory;
  end;

function GetLinkTarget(FileName: string): string;
function MakeLink(Link, Target: string): HRESULT;
function MakeSymlink(Link, Target: string): HRESULT;

implementation

{ TShellLink }

constructor TShellLink.Create(Parent: HWND);
begin
  inherited Create;

  CoInitialize(nil);
  FParentWnd := Parent;
  FShellLink := CreateComObject(CLSID_ShellLink) as IShellLink;
  FPersistFile := FShellLink as IPersistFile;
end;

destructor TShellLink.Destroy;
begin
  FPersistFile := nil;
  FShellLink := nil;
  CoUninitialize;

  inherited;
end;

function TShellLink.Resolve(Flags: Cardinal): Boolean;
var
  LastErr: HRESULT;
begin
  LastErr := FShellLink.Resolve(FParentWnd, Flags);
  SetLastError(LastErr);
  Result := LastErr = S_OK;
end;

function TShellLink.Load(FileName: string; Mode: Cardinal): Boolean;
var
  LastErr: HRESULT;
begin
  LastErr := FPerSistFile.Load(PWChar(FileName), Mode);
  SetLastError(LastErr);
  Result := LastErr = S_OK;
end;

function TShellLink.Save(FileName: string; Remember: Boolean): Boolean;
var
  LastErr: HRESULT;
begin
  LastErr := FPersistFile.Save(PChar(FileName), Remember);
  SetLastError(LastErr);
  Result := LastErr = S_OK;
end;

function TShellLink.ResolvingLoad(FileName: string; Mode, Flags: Cardinal): Boolean;
begin
  Result := Load(FileName, Mode);
  if Result then
    Result := Resolve(Flags);
end;

function TShellLink.ResolvingSave(FileName: string; Remember: Boolean; Flags: Cardinal): Boolean;
begin
  Result := Resolve(Flags);
  if Result then
    Result := Save(FileName, Remember);
end;

function TShellLink.GetArguments: string;
var
  Buffer: array[0..MAX_PATH] of Char;
begin
  FShellLink.GetArguments(Buffer, MAX_PATH);
  Result := string(Buffer);
end;

procedure TShellLink.SetArguments(Value: string);
begin
  FShellLink.SetArguments(PChar(Value));
end;

function TShellLink.GetDescription: string;
var
  Buffer: array[0..MAX_PATH] of Char;
begin
  FShellLink.GetDescription(Buffer, MAX_PATH);
  Result := string(Buffer);
end;

procedure TShellLink.SetDescription(Value: string);
begin
  FShellLink.SetDescription(PChar(Value));
end;

function TShellLink.GetHotkey: Word;
begin
  FShellLink.GetHotkey(Result);
end;

procedure TShellLink.SetHotkey(Value: Word);
begin
  FShellLink.SetHotkey(Value);
end;

function TShellLink.GetIconPath: string;
var
  Buffer: array[0..MAX_PATH] of Char;
  Index: Integer;
begin
  FShellLink.GetIconLocation(Buffer, MAX_PATH, Index);
  Result := string(Buffer);
end;

procedure TShellLink.SetIconPath(Value: string);
var
  Buffer: array[0..MAX_PATH] of Char;
  Index: Integer;
begin
  FShellLink.GetIconLocation(Buffer, MAX_PATH, Index);
  FShellLink.SetIconLocation(PChar(Value), Index);
end;

function TShellLink.GetIconIndex: Integer;
var
  Buffer: array[0..MAX_PATH] of Char;
  Index: Integer;
begin
  FShellLink.GetIconLocation(Buffer, MAX_PATH, Index);
  Result := Index;
end;

procedure TShellLink.SetIconIndex(Value: Integer);
var
  Buffer: array[0..MAX_PATH] of Char;
  Index: Integer;
begin
  FShellLink.GetIconLocation(Buffer, MAX_PATH, Index);
  FShellLink.SetIconLocation(Buffer, Value);
end;

function TShellLink.GetIDList: PItemIDList;
begin
  FShellLink.GetIDList(Result);
end;

procedure TShellLink.SetIDList(Value: PItemIDList);
begin
  FShellLink.SetIDList(Value);
end;

function TShellLink.GetTargetPath: string;
var
  FindData: TWin32FindData;
  Buffer: array[0..MAX_PATH] of Char;
begin
  FShellLink.GetPath(Buffer, MAX_PATH, FindData, SLGP_UNCPRIORITY);
  Result := string(Buffer);
end;

procedure TShellLink.SetTargetPath(Value: string);
begin
  FShellLink.SetPath(PChar(Value));
end;

function TShellLink.GetShowCmd: Integer;
begin
  FShellLink.GetShowCmd(Result);
end;

procedure TShellLink.SetShowCmd(Value: Integer);
begin
  FShellLink.SetShowCmd(Value);
end;

function TShellLink.GetWorkingDirectory: string;
var
  Buffer: array[0..MAX_PATH] of Char;
begin
  FShellLink.GetWorkingDirectory(Buffer, MAX_PATH);
  Result := string(Buffer);
end;

procedure TShellLink.SetWorkingDirectory(Value: string);
begin
  FShellLink.SetWorkingDirectory(PChar(Value));
end;

{ Routine }

{ Nt@CiV[gJbgjQƐ̃t@C𓾂
  FileName Nt@CłȂꍇAFileName ̂܂ܕԂA
  N悪Ȃꍇ͋󕶎Ԃ }
function GetLinkTarget(FileName: string): string;
var
  ShellLink: TShellLink;
begin
  if SameText('.lnk', ExtractFileExt(FileName)) then
  begin
    ShellLink := TShellLink.Create(0);
    try
      ShellLink.ResolvingLoad(FileName, STGM_READ, SLR_NO_UI);
      Result := ShellLink.TargetPath;
    finally
      ShellLink.Free;
    end;
  end
  else
    Result := FileName;
end;

{ TShellLink.Save ̊ȗ }
function MakeLink(Link, Target: string): HRESULT;
var
  ShellLink: TShellLink;
begin
  if not SameText('.lnk', ExtractFileExt(Link)) then
    Link := Link + '.lnk';

  ShellLink := TShellLink.Create(0);
  try
    with ShellLink do
    begin
      Arguments := '';
      TargetPath := Target;
      WorkingDirectory := ExtractFileDir(Target);
    end;
    if ShellLink.Save(Link, False) then
      Result := S_OK
    else
      Result := GetLastError;
  finally
    ShellLink.Free;
  end;
end;

{ gG[Ԃ FileCreateSymLink }
function MakeSymlink(Link, Target: string): HRESULT;
begin
  if FileCreateSymLink(Link, Target) then
    Result := S_OK
  else
    Result := GetLastError;
end;

end.
