(* ShellUtils

2012/02/01 ShellCtrls ̈ꕔ ShellProc 𓝍B
2016/08/02 HasSubItems CB
2020/10/02 ϐ` ShellTypes ɈړBقA낢CEύXEǉB
2020/03/13 ShellItem Manipulation ǉB

	VFl[Xy[X[eBeBW *)

unit ShellUtils platform;

interface

uses
  Winapi.Windows, System.SysUtils, System.Classes, System.Win.ComObj, Winapi.ShlObj, Winapi.ActiveX,
  Winapi.ShellAPI;

{ PIDL Manipulation }
function CreatePIDL(Size: Integer): PItemIDList;
function NextPIDL(IDList: PItemIDList): PItemIDList;
procedure StripLastID(IDList: PItemIDList);
function GetItemCount(IDList: PItemIDList): Integer;
function GetPIDLSize(IDList: PItemIDList): Integer;
function CopyPIDL(IDList: PItemIDList): PItemIDList;
function ConcatPIDLs(IDList1, IDList2: PItemIDList): PItemIDList;
procedure DisposePIDL(PIDL: PItemIDList);
function ExtractRelativeID(AbsoluteID: PItemIDList): PItemIDList;
function SameAbsoluteID(ID1, ID2: PItemIDList): Boolean;
function SameRelativeID(ParentFolder: IShellFolder; ID1, ID2: PItemIDList): Boolean;
function DesktopPIDL: PItemIDList;
function ParseDisplayName(Folder: IShellFolder; DisplayName: string): PItemIDList;
function GetDisplayName(PIDL: PItemIDList; DisplayForm: DWORD): string; overload;
function HasSubItems(PIDL: PItemIDList; Flags: Integer): Boolean; overload;
function GetPersistIDList(const punk: IUnknown; out ppidl: PItemIDList): HResult;
function GetVerbosePath(PIDL: PItemIDList): string; overload;
procedure ParseVerbosePath(Wnd: HWND; Path: string; var IDList: PItemIDList); overload;

{ Shell Folder Item Info }
function GetDisplayNameOf(ParentFolder: IShellFolder; PIDL: PItemIDList; Flags: Cardinal): string;
function GetFolderName(ParentFolder: IShellFolder; PIDL: PItemIDList): string;
function GetParseName(ParentFolder: IShellFolder; PIDL: PItemIDList): string;
function GetSpecialFolderPath(Folder: Integer): string; overload;
function GetSpecialFolderPath(Folder: TGUID): string; overload;
function GetShellImage(PIDL: PItemIDList; Large, Open: Boolean): Integer; overload;
function IsFolder(ParentFolder: IShellFolder; PIDL: PItemIDList): Boolean;
function IsDirectory(ParentFolder: IShellFolder; PIDL: PItemIDList): Boolean;
function HasSubFolders(ParentFolder: IShellFolder; PIDL: PItemIDList): Boolean;
function HasSubItems(ShellFolder: IShellFolder; Flags: Integer): Boolean; overload;

{ ShellItem Manipulation }
function DesktopItem: IShellItem;
function GetDisplayName(Item: IShellItem; DisplayForm: DWORD): string; overload;
function GetShellImage(Item: IShellItem; Large, Open: Boolean): Integer; overload;
function HasSubItems(Item: IShellItem; Flags: Integer): Boolean; overload;
function GetVerbosePath(Item: IShellItem): string; overload;
procedure ParseVerbosePath(Wnd: HWND; Path: string; var SHItem: IShellItem); overload;

{ Shell Library Item Info }
function IsLibrary(Item: IShellItem): Boolean; overload;
function IsLibrary(PIDL: PItemIDList): Boolean; overload;

implementation

uses
  ShellTypes, System.Masks;

{ PIDL Manipulation }

function CreatePIDL(Size: Integer): PItemIDList;
var
  Malloc: IMalloc;
begin
  OleCheck(SHGetMalloc(Malloc));

  Result := Malloc.Alloc(Size);
  if Assigned(Result) then
    FillChar(Result^, Size, 0);
end;

function NextPIDL(IDList: PItemIDList): PItemIDList;
begin
  Result := IDList;
  Inc(PByte(Result), IDList^.mkid.cb);
end;

procedure StripLastID(IDList: PItemIDList);
var
  MarkerID: PItemIDList;
begin
  MarkerID := IDList;
  if Assigned(IDList) then
  begin
    while IDList.mkid.cb <> 0 do
    begin
      MarkerID := IDList;
      IDList := NextPIDL(IDList);
    end;
    MarkerID.mkid.cb := 0;
  end;
end;

function GetItemCount(IDList: PItemIDList): Integer;
begin
  Result := 0;
  while IDList^.mkid.cb <> 0 do
  begin
    Inc(Result);
    IDList := NextPIDL(IDList);
  end;
end;

function GetPIDLSize(IDList: PItemIDList): Integer;
begin
  Result := 0;
  if Assigned(IDList) then
  begin
    Result := SizeOf(IDList^.mkid.cb);
    while IDList^.mkid.cb <> 0 do
    begin
      Result := Result + IDList^.mkid.cb;
      IDList := NextPIDL(IDList);
    end;
  end;
end;

function CopyPIDL(IDList: PItemIDList): PItemIDList;
var
  Size: Integer;
begin
  Size := GetPIDLSize(IDList);
  Result := CreatePIDL(Size);
  if Assigned(Result) then
    CopyMemory(Result, IDList, Size);
end;

function ConcatPIDLs(IDList1, IDList2: PItemIDList): PItemIDList;
var
  cb1, cb2: Integer;
begin
  if Assigned(IDList1) then
    cb1 := GetPIDLSize(IDList1) - SizeOf(IDList1^.mkid.cb)
  else
    cb1 := 0;

  cb2 := GetPIDLSize(IDList2);

  Result := CreatePIDL(cb1 + cb2);
  if Assigned(Result) then
  begin
    if Assigned(IDList1) then
      CopyMemory(Result, IDList1, cb1);
    CopyMemory(PByte(Result) + cb1, IDList2, cb2);
  end;
end;

procedure DisposePIDL(PIDL: PItemIDList);
begin
  if Assigned(PIDL) then
    CoTaskMemFree(PIDL);
end;

function ExtractRelativeID(AbsoluteID: PItemIDList): PItemIDList;
begin
  Result := CopyPIDL(AbsoluteID);
  while GetItemCount(Result) > 1 do
    Result := NextPIDL(Result);
  Result := CopyPIDL(Result);
end;

function SameAbsoluteID(ID1, ID2: PItemIDList): Boolean;
begin
  Result := DesktopShellFolder.CompareIDs(0, ID1, ID2) = 0;
end;

function SameRelativeID(ParentFolder: IShellFolder; ID1, ID2: PItemIDList): Boolean;
begin
  Result := ParentFolder.CompareIDs(0, ID1, ID2) = 0;
end;

function DesktopPIDL: PItemIDList;
begin
  OleCheck(SHGetFolderLocation(0, CSIDL_DESKTOP, 0, 0, Result));
end;

function ParseDisplayName(Folder: IShellFolder; DisplayName: string): PItemIDList;
var
  Chars: PWideChar;
  NumChars,
  Flags: LongWord;
begin
  Chars := StringToOleStr(DisplayName);
  try
    NumChars := Length(DisplayName);
    Flags := 0;
    OleCheck(Folder.ParseDisplayName(0, nil, Chars, NumChars, Result, Flags));
  finally
    SysFreeString(Chars);
  end;
end;

function GetDisplayName(PIDL: PItemIDList; DisplayForm: DWORD): string;
var
  DisplayName: PChar;
begin
  Result := '';
  DisplayName := nil;
  try
    SHGetNameFromIDList(PIDL, DisplayForm, DisplayName);
  finally
    if Assigned(DisplayName) then
    begin
      Result := DisplayName;
      CoTaskMemFree(DisplayName);
    end;
  end;
end;

function HasSubItems(PIDL: PItemIDList; Flags: Integer): Boolean;
var
  Item: IShellItem;
begin
  Result := False;
  //PIDL  DesktopPIDL ꍇADesktopShellFolder.BindToObject ̓[U[̃fXNgbvɂȂB
  if Succeeded(SHCreateShellItem(nil, nil, PIDL, Item)) then
    Result := HasSubItems(Item, Flags);
end;

function GetPersistIDList(const punk: IUnknown; out ppidl: PItemIDList): HResult;
var
  PersistID: IPersistIDList;
begin
  //SHGetIDListFromObject  Vista ȍ~̑ΉɂȂ̂ō쐬B
  Result := punk.QueryInterface(IID_IPersistIDList, PersistID);
  if Succeeded(Result) then
    Result := PersistID.GetIDList(ppidl);
end;

function GetVerbosePath(PIDL: PItemIDList): string;
var
  Item: IShellItem;
begin
  SHCreateShellItem(nil, nil, PIDL, Item);
  Result := GetVerbosePath(Item);
end;

//ĂȂǁAPIDL ̕ۑE ILSaveToStream, ILLoadFromStream g@邩B
procedure ParseVerbosePath(Wnd: HWND; Path: string; var IDList: PItemIDList);
const
  SHCONTF_ALLFILES = SHCONTF_FOLDERS or SHCONTF_NONFOLDERS or
                     SHCONTF_INCLUDEHIDDEN or SHCONTF_ENABLE_INCLUDESUPERHIDDEN or
                     SHCONTF_NAVIGATION_ENUM;
var
  List: TStrings;
  RelID: PItemIDList;
  ParentFolder: IShellFolder;
  ChildItem: IShellItem;
  i: Integer;
  s: string;
  EnumList: IEnumIDList;
  NumIDs: LongWord;
  PersistFolder: IPersistFolder2;
begin
  List := TStringList.Create;
  try
    List.Delimiter :='\';
    List.DelimitedText := Path;

    ParentFolder := DesktopShellFolder;
    for i := 0 to List.Count - 1 do
    begin
      ChildItem := nil;
      if Succeeded(ParentFolder.EnumObjects(Wnd, SHCONTF_ALLFILES, EnumList)) then
      begin
        while EnumList.Next(1, RelID, NumIDs) = S_OK do
        begin
          try
            SHCreateShellItem(nil, ParentFolder, RelID, ChildItem);
            s := GetDisplayName(ChildItem, SIGDN_PARENTRELATIVEPARSING);
            if List.Strings[i] = s then
            begin
              ChildItem.BindToHandler(nil, BHID_SFObject, IID_IShellFolder, ParentFolder);

              Break;
            end;
          finally
            DisposePIDL(RelID);
          end;
        end;
      end;
      if not Assigned(ChildItem) then
        Break;
    end;
    ParentFolder.QueryInterface(IID_IPersistFolder2, PersistFolder);
    PersistFolder.GetCurFolder(IDList);
  finally
    List.Free;
  end;
end;

{ Shell Folder Item Info }

function GetDisplayNameOf(ParentFolder: IShellFolder; PIDL: PItemIDList; Flags: Cardinal): string;
var
  StrRet: TStrRet;
begin
  Result := '';
  ParentFolder.GetDisplayNameOf(PIDL, Flags, StrRet);
  Result := StrRetToString(PIDL, StrRet);
end;

function GetFolderName(ParentFolder: IShellFolder; PIDL: PItemIDList): string;
begin
  Result := GetDisplayNameOf(ParentFolder, PIDL, SHGDN_INFOLDER);
end;

function GetParseName(ParentFolder: IShellFolder; PIDL: PItemIDList): string;
begin
  Result := GetDisplayNameOf(ParentFolder, PIDL, SHGDN_FORPARSING);
end;

function GetSpecialFolderPath(Folder: Integer): string;
var
  PIDL: PItemIDList;
begin
  try
    SHGetFolderLocation(0, Folder, 0, 0, PIDL);
    Result := GetParseName(DesktopShellFolder, PIDL);
  finally
    DisposePIDL(PIDL);
  end;
end;

function GetSpecialFolderPath(Folder: TGUID): string;
var
  ShellItem: IShellItem;
begin
  SHGetKnownFolderItem(Folder, 0, 0, IID_IShellItem, Pointer(ShellItem));
  Result := GetDisplayName(ShellItem, SIGDN_DESKTOPABSOLUTEPARSING);
end;

function GetShellImage(PIDL: PItemIDList; Large, Open: Boolean): Integer;
var
  FileInfo: TSHFileInfo;
  Flags: Integer;
begin
  Flags := SHGFI_PIDL or SHGFI_SYSICONINDEX;
  if Open then
    Flags := Flags or SHGFI_OPENICON;
  if Large then
    Flags := Flags or SHGFI_LARGEICON
  else
    Flags := Flags or SHGFI_SMALLICON;
  if SHGetFileInfo(PChar(PIDL), 0, FileInfo, SizeOf(FileInfo), Flags) > 0 then
    Result := FileInfo.iIcon
  else
    Result := -1;
end;

function IsFolder(ParentFolder: IShellFolder; PIDL: PItemIDList): Boolean;

  function IsCtrls(ShellFolder: IShellFolder; PIDL: PItemIDList): Boolean;
  var
    FileName,
    Mask: string;
  begin
    FileName := GetParseName(ShellFolder, PIDL);
    Mask := ExtractFileDir(GetSpecialFolderPath(CSIDL_CONTROLS)) + '*';
    Result := MatchesMask(FileName, Mask);
  end;

var
  ID: PItemIDList;
  Flags: LongWord;
  CurrentFolder: IShellFolder;
begin
  //PIDL IDꍇAɐ^Ԃ邽߁AIDɕϊĎsB
  if (ParentFolder = DesktopShellFolder) and (GetItemCount(PIDL) > 1) then
  begin
    ID := CopyPIDL(PIDL);
    try
      StripLastID(ID);
      DesktopShellFolder.BindToObject(ID, nil, IID_IShellFolder, ParentFolder);
    finally
      DisposePIDL(ID);
    end;
    ID := ExtractRelativeID(PIDL);
  end
  else
    ID := CopyPIDL(PIDL);
  try
    Flags := SFGAO_FOLDER;
    ParentFolder.GetAttributesOf(1, ID, Flags);
    Result := SFGAO_FOLDER and Flags <> 0;
    //Win7 (Vistaȍ~H) ł́AقƂǂ̃Rg[plڂɃtH_Ă邽߁A
    //Tuڂꍇ̂݁AtH_ƂĈ悤`FbNB
    if IsCtrls(ParentFolder, ID) then
    begin
      ParentFolder.BindToObject(ID, nil, IID_IShellFolder, CurrentFolder);
      Result := HasSubItems(CurrentFolder, SHCONTF_FOLDERS or SHCONTF_NONFOLDERS);
    end;
  finally
    DisposePIDL(ID);
  end;
end;

function IsDirectory(ParentFolder: IShellFolder; PIDL: PItemIDList): Boolean;
var
  ID: PItemIDList;
  Flags: LongWord;
begin
  if (ParentFolder = DesktopShellFolder) and (GetItemCount(PIDL) > 1) then
  begin
    ID := CopyPIDL(PIDL);
    try
      StripLastID(ID);
      DesktopShellFolder.BindToObject(ID, nil, IID_IShellFolder, ParentFolder);
    finally
      DisposePIDL(ID);
    end;
    ID := ExtractRelativeID(PIDL);
  end
  else
    ID := CopyPIDL(PIDL);
  try
    Flags := SFGAO_FOLDER or SFGAO_FILESYSANCESTOR;
    ParentFolder.GetAttributesOf(1, ID, Flags);
    Result := (SFGAO_FOLDER or SFGAO_FILESYSANCESTOR) and Flags <> 0;
  finally
    DisposePIDL(ID);
  end;
end;

function HasSubFolders(Parentfolder: IShellFolder; PIDL: PItemIDList): Boolean;
var
  Flags: LongWord;
begin
  Flags := SFGAO_CONTENTSMASK;
  ParentFolder.GetAttributesOf(1, PIDL, Flags);
  Result := SFGAO_HASSUBFOLDER and Flags <> 0;
end;

function HasSubItems(ShellFolder: IShellFolder; Flags: Integer): Boolean;
var
  ID: PItemIDList;
  EnumList: IEnumIDList;
  NumIDs: LongWord;
  HR: HResult;
  ErrMode: Integer;
begin
  Result := False;
  if Assigned(ShellFolder) then
  begin
    ErrMode := SetErrorMode(SEM_FAILCRITICALERRORS);
    try
      HR := ShellFolder.EnumObjects(0, Flags, EnumList);
      if HR <> S_OK then Exit;

      Result := EnumList.Next(1, ID, NumIDs) = S_OK;
    finally
      SetErrorMode(ErrMode);
    end;
  end;
end;

{ ShellItem Manipulation }

function DesktopItem: IShellItem;
var
  PIDL: PItemIDList;
begin
  PIDL := DesktopPIDL;
  try
    SHCreateShellItem(nil, nil, PIDL, Result);
  finally
    DisposePIDL(PIDL);
  end;
end;

function GetDisplayName(Item: IShellItem; DisplayForm: DWORD): string;
var
  DisplayName: PChar;
begin
  Result := '';
  DisplayName := nil;
  try
    Item.GetDisplayName(DisplayForm, DisplayName);
  finally
    if Assigned(DisplayName) then
    begin
      Result := DisplayName;
      CoTaskMemFree(DisplayName);
    end;
  end;
end;

function GetShellImage(Item: IShellItem; Large, Open: Boolean): Integer;
var
  PIDL: PItemIDList;
begin
  GetPersistIDList(Item, PIDL);
  try
    Result := GetShellImage(PIDL, Large, Open);
  finally
    DisposePIDL(PIDL);
  end;
end;

function HasSubItems(Item: IShellItem; Flags: Integer): Boolean;
var
  Folder: IShellFolder;
begin
  Result := False;
  if Succeeded(Item.BindToHandler(nil, BHID_SFObject, IID_IShellFolder, Folder)) then
    Result := HasSubItems(Folder, Flags);
end;

function GetVerbosePath(Item: IShellItem): string;
var
  List: TStrings;
  s: string;
begin
  List := TStringList.Create;
  try
    List.Delimiter :='\';

    while Assigned(Item) do
    begin
      s := GetDisplayName(Item, SIGDN_PARENTRELATIVEPARSING);
      List.Insert(0, s);
      Item.GetParent(Item);
    end;
    if List.Count > 0 then
      List.Delete(0);
      //List.Strings[0] := 'rfDesktop';

    Result := List.DelimitedText;
  finally
    List.Free;
  end;
end;

procedure ParseVerbosePath(Wnd: HWND; Path: string; var SHItem: IShellItem);
var
  PIDL: PItemIDList;
begin
  ParseVerbosePath(Wnd, Path, PIDL);
  try
    SHCreateShellItem(nil, nil, PIDL, SHItem);
  finally
    DisposePIDL(PIDL);
  end;
end;

{ Shell Library Item Info }

function IsLibrary(Item: IShellItem): Boolean;
begin
  Result := False;
  if CheckWin32Version(6, 1) then
    Result := Succeeded(ShellLibrary.LoadLibraryFromItem(Item, STGM_READ));
end;

function IsLibrary(PIDL: PItemIDList): Boolean;
var
  Item: IShellItem;
begin
  Result := False;
  if Succeeded(SHCreateShellItem(nil, nil, PIDL, Item)) then
    Result := IsLibrary(Item);
end;

end.
