unit DAA_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.DdeMan, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.ComCtrls,
  Vcl.WinXCtrls, AfViews, System.IOUtils;

type
  PFileAttributes = ^TFileAttributes;

  TFileProperty = record
    Creation: TDateTime;
    Modify: TDateTime;
    Access: TDateTime;
    Attrib: TFileAttributes;
    ZoneId: Boolean;
  end;
  PFileProperty = ^TFileProperty;

  TDateTimePicker = class(Vcl.ComCtrls.TDateTimePicker)
  private
    { Private 錾 }
    function GetDateTime: TDateTime;
    procedure SetDateTime(Value: TDateTime);
  public
    { Public 錾 }
    property DateTime: TDateTime read GetDateTime write SetDateTime;
  end;

  TMainForm = class(TForm)
    DdeServerConv: TDdeServerConv;
    SettingPanel: TPanel;
    OKButton: TButton;
    CancelButton: TButton;
    ApplyButton: TButton;
    PropertyPage: TPageControl;
    DateTimeTab: TTabSheet;
    CreationLbl: TLabel;
    CreationDatePic: TDateTimePicker;
    CreationTimePic: TDateTimePicker;
    ModifyLbl: TLabel;
    ModifyDatePic: TDateTimePicker;
    ModifyTimePic: TDateTimePicker;
    AccessLbl: TLabel;
    AccessDatePic: TDateTimePicker;
    AccessTimePic: TDateTimePicker;
    AttributeTab: TTabSheet;
    ReadonlyChk: TCheckBox;
    ReadonlyTgl: TToggleSwitch;
    HiddenChk: TCheckBox;
    HiddenTgl: TToggleSwitch;
    SystemChk: TCheckBox;
    SystemTgl: TToggleSwitch;
    ArchiveChk: TCheckBox;
    ArchiveTgl: TToggleSwitch;
    ZoneChk: TCheckBox;
    ResultView: TListViewAf;
    procedure FormCreate(Sender: TObject);
    procedure DdeServerConvExecuteMacro(Sender: TObject; Msg: TStrings);
    procedure OKButtonClick(Sender: TObject);
    procedure CancelButtonClick(Sender: TObject);
    procedure ApplyButtonClick(Sender: TObject);
    procedure AttribChksClick(Sender: TObject);
    procedure ResultViewDeletion(Sender: TObject; Item: TListItem);
    procedure ResultViewSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
  private
    { Private 錾 }
    procedure InitProperty(Item: TListItem);
    function FindFileProperty(FileName: string; var FindProp: TFileProperty): Boolean;
    function AddFile(FileName: string): TListItem;
    procedure ToggleAttribute(var Attribs: TFileAttributes;
      Check: TCheckBox; Switch: TToggleSwitch);
    function CorrectPickDay(OlgDay: TDateTime; NowDay: TDateTime;
      DatePicker, TimePicker: TDateTimePicker): TFileTime;
    procedure ApplyProperty(FileName: string);
    procedure RevertProperty(FileName: string);
    procedure WMDropFiles(var Msg: TWMDropFiles); message WM_DROPFILES;
  protected
    { protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  Winapi.ShellAPI, FileUtils, System.DateUtils;

{ Routine }

function AttribToStr(Attrib: TFileAttributes; ZoneId: Boolean): string;
begin
  Result := '';
  if TFileAttribute.faReadOnly in Attrib then Result := Result + 'R';
  if TFileAttribute.faHidden in Attrib then Result := Result + 'H';
  if TFileAttribute.faSystem in Attrib then Result := Result + 'S';
  if TFileAttribute.faArchive in Attrib then Result := Result + 'A';
  if ZoneId then Result := Result + 'Z';
end;

{ TDateTimePicker.DateTime C }

function TDateTimePicker.GetDateTime: TDateTime;
begin
  Result := inherited DateTime;
end;

procedure TDateTimePicker.SetDateTime(Value: TDateTime);
var
  OlgChecked: Boolean;
begin
  //DateTime ύX Checked = True ɂȂ̂ŁAŏB
  OlgChecked := Self.Checked;
  inherited DateTime := Value;
  Self.Checked := OlgChecked;
end;

{ TMainForm }

procedure TMainForm.InitProperty(Item: TListItem);
var
  Prop: TFileProperty;
begin
  if Assigned(Item) then
  begin
    Prop := PFileProperty(Item.Data)^;

    CreationDatePic.DateTime := Prop.Creation;
    CreationTimePic.DateTime := Prop.Creation;
    ModifyDatePic.DateTime := Prop.Modify;
    ModifyTimePic.DateTime := Prop.Modify;
    AccessDatePic.DateTime := Prop.Access;
    AccessTimePic.DateTime := Prop.Access;

    ReadonlyTgl.State := TToggleSwitchState(TFileAttribute.faReadOnly in Prop.Attrib);
    HiddenTgl.State := TToggleSwitchState(TFileAttribute.faHidden in Prop.Attrib);
    SystemTgl.State := TToggleSwitchState(TFileAttribute.faSystem in Prop.Attrib);
    ArchiveTgl.State := TToggleSwitchState(TFileAttribute.faArchive in Prop.Attrib);
  end
  else
  begin
    CreationDatePic.DateTime := Now;
    CreationTimePic.DateTime := CreationDatePic.DateTime;
    ModifyDatePic.DateTime := CreationDatePic.DateTime;
    ModifyTimePic.DateTime := CreationDatePic.DateTime;
    AccessDatePic.DateTime := CreationDatePic.DateTime;
    AccessTimePic.DateTime := CreationDatePic.DateTime;

    ReadonlyTgl.State := tssOff;
    HiddenTgl.State := tssOff;
    SystemTgl.State := tssOff;
    ArchiveTgl.State := tssOff;
  end;
end;

function TMainForm.FindFileProperty(FileName: string; var FindProp: TFileProperty): Boolean;
var
  Handle: THandle;
  Data: TWin32FindData;
begin
  Handle := Winapi.Windows.FindFirstFile(PChar(FileName), Data);
  try
    Result := Handle <> INVALID_HANDLE_VALUE;
    if Result then
    begin
      FindProp.Creation := FileTimeToLocalDateTime(Data.ftCreationTime);
      FindProp.Modify := FileTimeToLocalDateTime(Data.ftLastWriteTime);
      FindProp.Access := FileTimeToLocalDateTime(Data.ftLastAccessTime);
      FindProp.Attrib := TFile.IntegerToFileAttributes(Data.dwFileAttributes);
      FindProp.ZoneId := FileExists(FileName + ':Zone.Identifier');
    end;
  finally
    Winapi.Windows.FindClose(Handle);
  end;
end;

function TMainForm.AddFile(FileName: string): TListItem;
var
  Prop: PFileProperty;
begin
  New(Prop);
  if FindFileProperty(FileName, Prop^) then
  begin
    Result := ResultView.Items.Add;
    Result.Caption := FileName;
    Result.SubItems.Add(DateTimeToStr(Prop^.Creation));
    Result.SubItems.Add(DateTimeToStr(Prop^.Modify));
    Result.SubItems.Add(DateTimeToStr(Prop^.Access));
    Result.SubItems.Add(AttribToStr(Prop^.Attrib, Prop^.ZoneId));
    Result.Data := Pointer(Prop);
  end
  else
  begin
    Dispose(Prop);
    Result := nil;
  end;
end;

procedure TMainForm.ToggleAttribute(var Attribs: TFileAttributes;
  Check: TCheckBox; Switch: TToggleSwitch);
begin
  if Check.Checked then
    case Switch.State of
      tssOff: Exclude(Attribs, TFileAttribute(Switch.Tag));
      tssOn: Include(Attribs, TFileAttribute(Switch.Tag));
    end;
end;

function TMainForm.CorrectPickDay(OlgDay: TDateTime; NowDay: TDateTime;
  DatePicker, TimePicker: TDateTimePicker): TFileTime;
begin
  if DatePicker.Checked then
    ReplaceDate(OlgDay, DatePicker.Date);
  if TimePicker.Checked then
    ReplaceTime(OlgDay, TimePicker.Time);
  if SameDateTime(OlgDay, NowDay) then
    ZeroFileTime(Result)
  else
    Result := LocalDateTimeToFileTime(OlgDay);
end;

procedure TMainForm.ApplyProperty(FileName: string);
var
  Item: TListItem;
  OlgProp, NowProp: TFileProperty;
  CreationTime, ModifyTime, AccessTime: TFileTime;
  Attr: TFileAttributes;
begin
  Item := ResultView.FindCaption(0, FileName, False, True, False);
  if not Assigned(Item) then
    Item := AddFile(FileName);
  if Assigned(Item) then
  begin
    OlgProp := PFileProperty(Item.Data)^;
    if ZoneChk.Checked and OlgProp.ZoneId then
    begin
      DeleteFile(FileName + ':Zone.Identifier');
      PFileProperty(Item.Data)^.ZoneId := FileExists(FileName + ':Zone.Identifier');
    end;
    if FindFileProperty(FileName, NowProp) then
    begin
      Attr := OlgProp.Attrib;
      ToggleAttribute(Attr, ReadonlyChk, ReadonlyTgl);
      ToggleAttribute(Attr, HiddenChk, HiddenTgl);
      ToggleAttribute(Attr, SystemChk, SystemTgl);
      ToggleAttribute(Attr, ArchiveChk, ArchiveTgl);
      if NowProp.Attrib <> Attr then
        SetFileAttributes(PChar(FileName), TFile.FileAttributesToInteger(Attr));
    end;
    if FindFileProperty(FileName, NowProp) then
    begin
      CreationTime := CorrectPickDay(OlgProp.Creation, NowProp.Creation, CreationDatePic, CreationTimePic);
      ModifyTime := CorrectPickDay(OlgProp.Modify, NowProp.Modify, ModifyDatePic, ModifyTimePic);
      AccessTime := CorrectPickDay(OlgProp.Access, NowProp.Access, AccessDatePic, AccessTimePic);
      SetTimeStamp(FileName, @CreationTime, @AccessTime, @ModifyTime);
    end;
    if FindFileProperty(FileName, NowProp) then
    begin
      Item.SubItems.Strings[0] := DateTimeToStr(NowProp.Creation);
      Item.SubItems.Strings[1] := DateTimeToStr(NowProp.Modify);
      Item.SubItems.Strings[2] := DateTimeToStr(NowProp.Access);
      Item.SubItems.Strings[3] := AttribToStr(NowProp.Attrib, NowProp.ZoneId);
    end
  end;
end;

procedure TMainForm.RevertProperty(FileName: string);
var
  Item: TListItem;
  OlgProp, NowProp: TFileProperty;
  CreationTime, ModifyTime, AccessTime: TFileTime;
begin
  Item := ResultView.FindCaption(0, FileName, False, True, False);
  if Assigned(Item) then
  begin
    OlgProp := PFileProperty(Item.Data)^;
    if FindFileProperty(FileName, NowProp) then
    begin
      if NowProp.Attrib <> OlgProp.Attrib then
        SetFileAttributes(PChar(FileName), TFile.FileAttributesToInteger(OlgProp.Attrib));
    end;
    if FindFileProperty(FileName, NowProp) then
    begin
      if NowProp.Creation <> OlgProp.Creation then
        CreationTime := LocalDateTimeToFileTime(OlgProp.Creation)
      else
        ZeroFileTime(CreationTime);
      if NowProp.Modify <> OlgProp.Modify then
        ModifyTime := LocalDateTimeToFileTime(OlgProp.Modify)
      else
        ZeroFileTime(ModifyTime);
      if NowProp.Access <> OlgProp.Access then
        AccessTime := LocalDateTimeToFileTime(OlgProp.Access)
      else
        ZeroFileTime(AccessTime);
      SetTimeStamp(FileName, @CreationTime, @AccessTime, @ModifyTime);
    end;
  end;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited;
  //t@Chbv󂯓B
  Params.ExStyle := Params.EXStyle or WS_EX_ACCEPTFILES;
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  {$IFDEF RELEASE}
  PropertyPage.TabIndex := 0;
  {$ENDIF RELEASE}

  for i := 1 to ParamCount do
    ApplyProperty(ParamStr(i));
  InitProperty(nil);
end;

procedure TMainForm.WMDropFiles(var Msg: TWMDropFiles);
var
  Count, i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  ResultView.Items.Clear;

  Count := DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0);
  try
    for i := 0 to Count - 1 do
    begin
      DragQueryFile(Msg.Drop, i, FileName, SizeOf(FileName));
      ApplyProperty(FileName);
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TMainForm.DdeServerConvExecuteMacro(Sender: TObject; Msg: TStrings);
var
  i: Integer;
begin
  ResultView.Items.Clear;

  for i := 0 to Msg.Count - 1 do
    ApplyProperty(Msg.Strings[i]);
end;

procedure TMainForm.OKButtonClick(Sender: TObject);
begin
  ApplyButton.Click;
  Close;
end;

procedure TMainForm.CancelButtonClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to ResultView.Items.Count - 1 do
    RevertProperty(ResultView.Items.Item[i].Caption);

  Close;
end;

procedure TMainForm.ApplyButtonClick(Sender: TObject);
var
  i: Integer;
begin
  for i := 0 to ResultView.Items.Count - 1 do
    ApplyProperty(ResultView.Items.Item[i].Caption);
end;

procedure TMainForm.AttribChksClick(Sender: TObject);
var
  Chk: TCheckBox;
  Tgl: TToggleSwitch;
begin
  Chk := Sender as TCheckBox;
  Tgl := FindComponent(Copy(Chk.Name, 1, Length(Chk.Name) - 3) + 'Tgl') as TToggleSwitch;
  if Assigned(Tgl) then
    Tgl.Enabled := Chk.Checked;
end;

procedure TMainForm.ResultViewDeletion(Sender: TObject; Item: TListItem);
begin
  Dispose(PFileProperty(Item.Data));
end;

procedure TMainForm.ResultViewSelectItem(Sender: TObject; Item: TListItem; Selected: Boolean);
begin
  InitProperty(ResultView.Selected)
end;

end.
