unit RWP_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.Actions, Vcl.ActnList, Vcl.StdCtrls, Vcl.ExtCtrls,
  AfCombo, RefCtrls;

type
  TMainForm = class(TForm)
    ActionList: TActionList;
    OpenAct: TAction;
    CancelAct: TAction;
    RunAsAct: TAction;
    OpenButton: TButton;
    CancelButton: TButton;
    RunAsButton: TButton;
    InfoPanel: TPanel;
    FileLbl: TLabel;
    FileEdt: TComboBoxAf;
    FileBtn: TRefFileButton;
    ParametersLbl: TLabel;
    ParametersEdt: TComboBoxAf;
    ParametersBtn: TRefFileButton;
    DirectoryLbl: TLabel;
    DirectoryEdt: TComboBoxAf;
    DirectoryBtn: TRefDirectoryButton;
    ShowLbl: TLabel;
    ShowCbx: TComboBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ExecuteActsUpdate(Sender: TObject);
    procedure ExecuteActsExecute(Sender: TObject);
    procedure CancelActExecute(Sender: TObject);
    procedure FileBtnExecute(Sender: TObject);
    procedure ParametersBtnExecute(Sender: TObject);
    procedure DirectoryBtnExecute(Sender: TObject);
  private
    { Private 錾 }
    FModified: Boolean;
    procedure ReadOption;
    procedure WriteOption;
    procedure UpdateInfo(FileName: string);
    procedure ApplicationIdleAtOnce(Sender: TObject; var Done: Boolean);
    procedure WMDropFiles(Var Msg: TWMDropFiles); message WM_DROPFILES;
    function SuggestCloseUp: Boolean;
  protected
    { protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  AppData, System.StrUtils, ShellLink, Winapi.ShellAPI;

procedure TMainForm.ReadOption;
var
  IniFile: TAppIniFile;
  Section: TStrings;
  i: Integer;
begin
  IniFile := TAppIniFile.Create(AppDataName('ShaggyDog\RightOn\RunWithParameter.ini'));
  try
    Section := TStringList.Create;
    try
      //File
      IniFile.ReadSectionValues('File', Section);
      FileEdt.Items.Clear;
      for i := Section.Count - 1 downto 0 do
        FileEdt.PushHistory(Section.ValueFromIndex[i]);
      //Parameters
      IniFile.ReadSectionValues('Parameters', Section);
      ParametersEdt.Items.Clear;
      for i := Section.Count - 1 downto 0 do
        ParametersEdt.PushHistory(Section.ValueFromIndex[i]);
      //Directory
      IniFile.ReadSectionValues('Directory', Section);
      DirectoryEdt.Items.Clear;
      for i := Section.Count - 1 downto 0 do
        DirectoryEdt.PushHistory(Section.ValueFromIndex[i]);
    finally
      Section.Free;
    end;
  finally
    IniFile.Free;
  end;
  FModified := False;
end;

procedure TMainForm.WriteOption;
var
  IniFile: TAppIniFile;
  i: Integer;
begin
  IniFile := TAppIniFile.Create(AppDataName('ShaggyDog\RightOn\RunWithParameter.ini'));
  try
    //File
    IniFile.DeleteSection('File');
    for i := 0 to FileEdt.Items.Count - 1 do
      IniFile.WriteQuotedString('File', IntToStr(i), FileEdt.Items.Strings[i]);
    //Parameters
    IniFile.DeleteSection('Parameters');
    for i := 0 to ParametersEdt.Items.Count - 1 do
      IniFile.WriteQuotedString('Parameters', IntToStr(i), ParametersEdt.Items.Strings[i]);
    //Directory
    IniFile.DeleteSection('Directory');
    for i := 0 to DirectoryEdt.Items.Count - 1 do
      IniFile.WriteQuotedString('Directory', IntToStr(i), DirectoryEdt.Items.Strings[i]);
  finally
    IniFile.Free;
  end;
  FModified := False;
end;

procedure TMainForm.UpdateInfo(FileName: string);
var
  PathExt: string;
  IsFile: Boolean;
begin
  IsFile := False;
  for PathExt in SplitString(GetEnvironmentVariable('PATHEXT'), ';') do
  begin
    IsFile := SameText(PathExt, ExtractFileExt(FileName));
    if IsFile then
    begin
      FileEdt.SetFocus;
      FileEdt.Text := FileName;

      Break;
    end;
  end;

  if not IsFile then
  begin
    ParametersEdt.SetFocus;
    ParametersEdt.Text := FileName;
  end;
end;

procedure TMainForm.ApplicationIdleAtOnce(Sender: TObject; var Done: Boolean);
var
  i: Integer;
begin
  if Assigned(Application.OnIdle) then
  begin
    Application.OnIdle := nil;

    for i := 1 to ParamCount do
      UpdateInfo(GetLinkTarget(ParamStr(i)));
    OpenButton.SetFocus;
  end;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  ReadOption;
  RunAsButton.Visible := TOSVersion.Check(6);

  if ParamCount > 0 then
    Application.OnIdle := ApplicationIdleAtOnce;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  //t@Chbv󂯓B
  Params.ExStyle := Params.EXStyle or WS_EX_ACCEPTFILES;
end;

procedure TMainForm.WMDropFiles(Var Msg: TWMDropFiles);
var
  i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  //t@ChbvB
  try
    for i := 0 to DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0) - 1 do
    begin
      DragQueryFile(Msg.Drop, i, FileName, SizeOf(FileName));
      UpdateInfo(GetLinkTarget(FileName));
    end;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if FModified then
    WriteOption;
end;

function TMainForm.SuggestCloseUp: Boolean;
var
  OlgActive: TWinControl;
begin
  Result := FileEdt.SuggestDropedDown or ParametersEdt.SuggestDropedDown or DirectoryEdt.SuggestDropedDown;

  if Result then
  begin
    OlgActive := ActiveControl;
    try
      ActiveControl := nil;
    finally
      ActiveControl := OlgActive;
    end;
  end;
end;

procedure TMainForm.ExecuteActsUpdate(Sender: TObject);
begin
  TAction(Sender).Enabled := FileEdt.Text <> '';
end;

procedure TMainForm.ExecuteActsExecute(Sender: TObject);
var
  ExecuteInfo: TShellExecuteInfo;
begin
  if not SuggestCloseUp then
  begin
    {$B+}
    if FileEdt.UpdateHistory or ParametersEdt.UpdateHistory or DirectoryEdt.UpdateHistory then
      FModified := True;
    {$B-}

    FillChar(ExecuteInfo, SizeOf(TShellExecuteInfo), 0);
    with ExecuteInfo do
    begin
      cbSize := SizeOf(TShellExecuteInfo);
      fMask := SEE_MASK_DOENVSUBST;
      wnd := Handle;
      if Sender = OpenAct then
        lpVerb := 'open'
      else
        lpVerb := 'runas';
      nShow := SW_SHOWNORMAL;
      lpFile := PChar(FileEdt.Text);
      lpParameters := PChar(ParametersEdt.Text);
      lpDirectory := PChar(DirectoryEdt.Text);
      case ShowCbx.ItemIndex of
        0: nShow := SW_SHOWNORMAL;
        1: nShow := SW_MINIMIZE;
        2: nShow := SW_MAXIMIZE;
      end;
    end;
    if ShellExecuteEx(@ExecuteInfo) then
      Close;
  end;
end;

procedure TMainForm.CancelActExecute(Sender: TObject);
begin
  if not SuggestCloseUp then
    Close;
end;

procedure TMainForm.FileBtnExecute(Sender: TObject);
begin
  FileEdt.Text := FileBtn.Dialog.FileName;
end;

procedure TMainForm.ParametersBtnExecute(Sender: TObject);
begin
  ParametersEdt.Text := ParametersBtn.Dialog.FileName;
end;

procedure TMainForm.DirectoryBtnExecute(Sender: TObject);
begin
  DirectoryEdt.Text := DirectoryBtn.Dialog.Directory;
end;

end.
