unit STC_Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.DdeMan, System.ImageList, Vcl.ImgList,
  Vcl.PlatformDefaultStyleActnCtrls, System.Actions, Vcl.ActnList, Vcl.ActnMan, Vcl.ToolWin,
  Vcl.ActnCtrls, Vcl.ComCtrls, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  TMainForm = class(TForm)
    DdeServerConv: TDdeServerConv;
    ActionImages: TImageList;
    ActionManager: TActionManager;
    CopyAct: TAction;
    NamesAct: TAction;
    LinesAct: TAction;
    ActionToolBar: TActionToolBar;
    OptionPage: TPageControl;
    NamesTab: TTabSheet;
    NameOnlyChk: TCheckBox;
    ExcludeExtChk: TCheckBox;
    ShortNameChk: TCheckBox;
    SeparatorGrp: TRadioGroup;
    LinesTab: TTabSheet;
    InsertNameGrp: TRadioGroup;
    StatusBar: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure DdeServerConvExecuteMacro(Sender: TObject; Msg: TStrings);
    procedure CopyActUpdate(Sender: TObject);
    procedure CopyActExecute(Sender: TObject);
    procedure NamesActExecute(Sender: TObject);
    procedure LinesActExecute(Sender: TObject);
  private
    { Private 錾 }
    FSources: TStringList;
    procedure UpdateStatusBar;
    procedure WMDropFiles(Var Msg: TWMDropFiles); message WM_DROPFILES;
    procedure SendFileNames;
    procedure SendFileContents;
  protected
    { protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public 錾 }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses
  Winapi.ShellAPI, Vcl.Clipbrd, Winapi.MLang, MLangUtils;

resourcestring
  SItemsCount = '%d';


{ Routine }

// t@Cpň͂ށB
function QuotedFileName(FileName: string): string;
begin
  Result := FileName;
  if Pos(' ', Result) > 0 then
    Result := AnsiQuotedStr(Result, '"');
end;

{ TMainForm }

procedure TMainForm.UpdateStatusBar;
begin
  StatusBar.Panels.Items[0].Text := Format(SItemsCount, [FSources.Count]);
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  {$IFDEF RELEASE}
  OptionPage.TabIndex := 0;
  {$ENDIF RELEASE}

  FSources := TStringList.Create;
  FSources.Sorted := True;

  for i := 1 to ParamCount do
    FSources.Add(ParamStr(i));
  UpdateStatusBar;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  //t@Chbv󂯓B
  Params.ExStyle := Params.EXStyle or WS_EX_ACCEPTFILES;
end;

procedure TMainForm.WMDropFiles(Var Msg: TWMDropFiles);
var
  i: Integer;
  FileName: array[0..MAX_PATH] of Char;
begin
  //t@ChbvB
  FSources.Clear;
  try
    for i := 0 to DragQueryFile(Msg.Drop, $FFFFFFFF, nil, 0) - 1 do
    begin
      DragQueryFile(Msg.Drop, i, FileName, SizeOf(FileName));
      FSources.Add(FileName);
    end;
    UpdateStatusBar;
  finally
    DragFinish(Msg.Drop);
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FSources.Free;
end;

procedure TMainForm.DdeServerConvExecuteMacro(Sender: TObject; Msg: TStrings);
begin
  FSources.Assign(Msg);
  UpdateStatusBar;
end;

procedure TMainForm.SendFileNames;
var
  i: Integer;
  s, Names: string;
begin
  for i := 0 to FSources.Count - 1 do
  begin
    s := FSources.Strings[i];
    if ShortNameChk.Checked then
      s := ExtractShortPathName(s);
    if NameOnlyChk.Checked then
      s := ExtractFileName(s);
    if ExcludeExtChk.Checked then
      s := ChangeFileExt(s, '');
    case SeparatorGrp.ItemIndex of
      0: Names := Names + s + ';';
      1: Names := Names + s + #13#10;
      2: Names := Names + QuotedFileName(s) + ' ';
    end;
  end;
  case SeparatorGrp.ItemIndex of
    0, 2: Names := Copy(Names, 1, Length(Names) - 1);
    //! 1: Names := Copy(Names, 1, Length(Names) - 2);
  end;
  Clipboard.SetTextBuf(PChar(Names));
end;

procedure TMainForm.SendFileContents;
var
  AllList, SubList: TStrings;
  FileName: string;
  FS: TFileStream;
  ENC: TEncoding;
  i: Integer;
begin
  AllList := TStringList.Create;
  try
    for i := 0 to FSources.Count - 1 do
    begin
      FileName := ExtractFileName(FSources.Strings[i]);
      case InsertNameGrp.ItemIndex of
        1, 3: AllList.Add(FileName);
        4: AllList.Add('-^ ' + QuotedFileName(FileName));
      end;

      FS := TFileStream.Create(FSources.Strings[i], fmOpenRead);
      try
        ENC := GetEncodingInstance(DetectCodepage(MLDETECTCP_NONE, 0, FS));
        try
          SubList := TStringList.Create;
          try
            SubList.LoadFromStream(FS, ENC);
            AllList.AddStrings(SubList);
          finally
            SubList.Free;
          end;
        finally
          ENC.Free;
        end;
      finally
        FS.Free;
      end;

      case InsertNameGrp.ItemIndex of
        2, 3: AllList.Add(FileName);
        4: AllList.Add('-$ ' + QuotedFileName(FileName));
      end;
    end;
    Clipboard.SetTextBuf(PChar(AllList.Text));
  finally
    AllList.Free;
  end;
end;

procedure TMainForm.CopyActUpdate(Sender: TObject);
begin
  CopyAct.Enabled := FSources.Count > 0;
end;

procedure TMainForm.CopyActExecute(Sender: TObject);
begin
  if NamesAct.Checked then
    SendFileNames
  else if LinesAct.Checked then
    SendFileContents;
  Close;
end;

procedure TMainForm.NamesActExecute(Sender: TObject);
begin
  OptionPage.ActivePage := NamesTab;
end;

procedure TMainForm.LinesActExecute(Sender: TObject);
begin
  OptionPage.ActivePage := LinesTab;
end;

end.
